<?php

class GJMAA_Model
{

    protected $wpdb;

    protected $prefix;

    private $dbVersion = '2.0.0';

    protected $_data;

    protected $columns;

    protected $defaultPK = 'id';

    protected $tableName;

    public function __construct()
    {
        global $wpdb;
        $this->prefix = $wpdb->prefix;
        $this->wpdb = $wpdb;
    }

    public function getTable()
    {
        return $this->prefix . $this->tableName;
    }

    public function createTable($columns, $primaryKey = null)
    {
        $tableName = $this->getTable();
        if ($this->wpdb->get_var("SHOW TABLES LIKE '" . $tableName . "'") != $tableName) {
            $sql = "CREATE TABLE " . $tableName . " (";
            foreach ($columns as $name => $attributes) {
                $sql .= $name . " " . implode(" ", $attributes['schema']) . ',';
            }
            $sql .= $primaryKey ? ' PRIMARY KEY (' . $primaryKey . ')' : '';
            $sql = rtrim($sql, ',');
            $sql .= ") CHARACTER SET utf8 COLLATE utf8_bin;";

            $this->wpdb->query($sql);
        }
    }

    public function install()
    {
        $this->createTable($this->getColumns(), $this->getDefaultPk());
    }

    public function uninstall()
    {
        $this->wpdb->query("DROP TABLE " . $this->getTable());
    }

    public function getData($key = null)
    {
        return $key ? (isset($this->_data[$key]) ? $this->_data[$key] : null) : $this->_data;
    }

    public function setData($key, $value = null)
    {
        if (! is_array($key)) {
            $this->_data[$key] = $value;
        } else {
            $this->_data = $key;
        }

        return $this;
    }

    public function unsetData()
    {
        $this->_data = [];
    }

    public function getColumn($name)
    {
        return $this->columns[$name];
    }
    
    public function addColumn($table, $name, $schema){
        $this->wpdb->query(
            sprintf(
                'ALTER TABLE %s ADD %s %s',
                $table,
                $name,
                implode(' ',$schema)
            )
        );
    }
    
    public function removeColumn($table, $name){
        $this->wpdb->query(
            sprintf(
                'ALTER TABLE %s DROP COLUMN %s', $table, $name
            )
        );
    }

    public function getColumns()
    {
        return $this->columns;
    }

    public function setColumns($columns = [])
    {
        $this->columns = $columns;
        return $this;
    }
    
    public function migrate(){
        $data = $this->getData();
        $this->migrateData($data);
    }

    public function save()
    {
        $data = $this->getData();
        if (isset($data[$this->getDefaultPk()]) && $data[$this->getDefaultPk()] != 0) {
            $id = $data[$this->getDefaultPk()];
            unset($data[$this->getDefaultPk()]);
            $this->updateData($id, $data);
        } else {
            unset($data[$this->getDefaultPk()]);
            $this->insertData($data);
        }

        return $this;
    }

    public function delete()
    {
        if ($this->getData($this->getDefaultPk())) {
            $this->wpdb->query('DELETE FROM ' . $this->getTable() . ' WHERE ' . $this->getDefaultPk() . ' = ' . $this->getData($this->getDefaultPk()));
        }

        $this->unsetData();

        return $this;
    }

    public function getDefaultPk()
    {
        return $this->defaultPK;
    }

    public function setDefaultPk($key = 'id')
    {
        $this->defaultPK = $key;
    }

    public function insertData($data)
    {
        $this->wpdb->insert($this->getTable(), $data, $this->parseFormat($data));

        return $this->wpdb->insert_id ? $this->load($this->wpdb->insert_id) : $this->unsetData();
    }
    
    public function migrateData($data){
        $this->wpdb->insert($this->getTable(), $data, $this->parseFormat($data));
        
        if(!$this->wpdb->last_error)
            return;
        
        $id = $data[$this->getDefaultPk()];
        unset($data[$this->getDefaultPk()]);
        
        $this->updateData($id,$data);
    }

    public function updateData($id, $data)
    {
        $whereData = [
            $this->getDefaultPk() => $id
        ];

        $this->wpdb->update($this->getTable(), $data, $whereData, $this->parseFormat($data), $this->parseFormat($whereData));

        return $this->load($id);
    }

    public function parseFormat($data)
    {
        $format = [];
        foreach ($data as $index => $value) {
            $column = $this->getColumn($index);
            $format[] = $column['format'];
        }

        return $format;
    } 

    public function load($value, $column = null)
    {
        $columns = is_null($column) ? [
            $this->getDefaultPk()
        ] : (! is_array($column) ? [
            $column
        ] : $column);

        $values = ! is_array($value) ? [
            $value
        ] : $value;
        $where = [];

        foreach ($columns as $index => $column) {
            $where[] = sprintf('%s="%s"', $column, $values[$index]);
        }
        $sql = sprintf("SELECT * FROM %s WHERE %s", $this->getTable(), implode(' AND ', $where));

        $response = $this->wpdb->get_row($sql, ARRAY_A);

        if (! empty($response)) {
            foreach ($response as $key => $data) {
                $this->setData($key, $data);
            }
        } else {
            $this->unsetData();
        }

        return $this;
    }

    public function getAll()
    {
        $sql = "SELECT * FROM " . $this->getTable();
        return $this->wpdb->get_results($sql, ARRAY_A);
    }
    
    public function getAllIds(){
        $all = $this->getAll();
        
        $ids = [];
        
        foreach($all as $entity){
            $ids[] = $entity[$this->getDefaultPk()];
        }
        
        return $ids;
    }

    public function getCountAll()
    {
        $sql = "SELECT COUNT(*) FROM " . $this->getTable();
        return $this->wpdb->get_var($sql);
    }

    public function findIn($field, $array)
    {
        $query = 'SELECT * FROM ' . $this->getTable();
        if (is_array($array)) {
            $query .= " WHERE $field IN ('" . implode("','", $array) . "')";
        } else {
            $query .= " WHERE $field = '$array'";
        }

        return $this->wpdb->get_results($query, ARRAY_A);
    }

    public function getId()
    {
        return isset($this->_data[$this->defaultPK]) ? $this->_data[$this->defaultPK] : null;
    }

    public function update($currentVersion)
    {
        return false;
    }

    public function searchParseToWhere($search, $columns)
    {
        if (is_null($search) || $search === '' || (!is_numeric($search) && empty($search))) {
            return '';
        }
        
        $search = strtolower($search);
        $where = [];

        foreach ($columns as $columnName) {
            if (! isset($this->columns[$columnName])) {
                continue;
            }

            $format = $this->columns[$columnName]['format'];

            switch ($format) {
                case '%d':
                case '%f':
                    if(is_numeric($search)){
                    $where[] = $columnName . ' = ' . $search;
                    }
                    break;
                case '%s':
                default:
                    $where[] = $this->wpdb->prepare('LOWER(' . $columnName . ') LIKE %s', '%' . $search . '%');
                    break;
            }
        }

        return empty($where) ? '' : 'WHERE ' . implode(' OR ', $where);
    }

    public function getCountFilteredResult($search, $columns)
    {
        $where = $this->searchParseToWhere($search, $columns);

        $query = "SELECT COUNT(*) FROM " . $this->getTable() . " " . $where;

        return $this->wpdb->get_var($query);
    }

    public function getFilteredResult($page, $limit, $search, $columns, $sort)
    {
        $offset = $page * $limit - $limit;

        $where = $this->searchParseToWhere($search, $columns);

        $query = $this->wpdb->prepare("SELECT * FROM " . $this->getTable() . " " . $where . " " . $sort . " LIMIT %d OFFSET %d", $limit, $offset);
        
        return $this->wpdb->get_results($query, ARRAY_A);
    }
    
    public function isJson($string){
        json_decode($string);
        return (json_last_error() == JSON_ERROR_NONE);
    }
}