<?php

class GJMAA_Cron_Woocommerce_New
{

    public function execute()
    {
        $wooCommerceService = GJMAA::getService('woocommerce');
        if (! $wooCommerceService->isEnabled()) {
            return;
        }

        $profileModel = GJMAA::getModel('profiles');
        $wooCommerceProfileIds = $profileModel->getWooCommerceProfileIds();

        if (empty($wooCommerceProfileIds)) {
            return;
        }

        $filters = [
            'WHERE' => sprintf('auction_profile_id IN (%s) AND (auction_in_woocommerce IS NULL OR auction_in_woocommerce = 0)', implode(',', $wooCommerceProfileIds)),
            'LIMIT' => 10
        ];

        $auctionsModel = GJMAA::getModel('auctions');
        $auctions = $auctionsModel->getAllBySearch($filters);

        if (empty($auctions)) {
            return;
        }

        $auctionIds = [];
        foreach ($auctions as $auction) {
            $auctionIds[] = $auction['auction_id'];
        }

        if (count($auctionIds) == 0) {
            return;
        }

        $profileModel->unsetData();
        $profile = $profileModel->load($wooCommerceProfileIds[0]);

        $restApiImport = GJMAA::getService('import');
        $restApiImport->setProfile($profile);
        $isRestConnected = $restApiImport->connect();
        $webapiImport = null;
        if ($isRestConnected) { 
            $webapiImport = $restApiImport->connectToWebAPI();
        } else { 
            $webApiImportService = GJMAA::getService('webapi_import');
            $webApiImportService->setProfile($profile);
            $isConnected = $webApiImportService->connect();
            if (! $isConnected) { 
                return;
            } 
            $webapiImport = $webApiImportService->getClient();
        }

        $response = $webapiImport->getItemAuction($auctionIds);
        if ($message = $webapiImport->getError()) {
            error_log($message);
            return;
        }

        $items = $response->arrayItemListInfo->item;
        $auctionDetails = is_array($items) ? $items : [
            $items
        ];

        $serviceWooCommerce = GJMAA::getService('woocommerce');
        $serviceWooCommerce->saveProducts($auctionDetails);

        foreach ($auctions as $auction) {
            $auctionsModel->unsetData();

            $auction['auction_in_woocommerce'] = 1;

            $auctionsModel->setData($auction);
            $auctionsModel->save();
        }
    }

    public static function run()
    {
        (new self())->execute();
    }
}
?>