<?php

class GJMAA_Helper_Settings
{

    public function getFieldsData()
    {
        return [
            'setting_id' => [
                'id' => 'setting_id',
                'type' => 'hidden',
                'name' => 'setting_id'
            ],
            'setting_name' => [
                'id' => 'setting_name',
                'type' => 'text',
                'name' => 'setting_name',
                'label' => 'Name',
                'help' => 'Custom name of your API settings',
                'required' => true
            ],
            'setting_site' => [
                'id' => 'setting_site',
                'type' => 'select',
                'name' => 'setting_site',
                'label' => 'Setting Site',
                'source' => 'allegro_site',
                'help' => 'Choose auction site',
                'required' => true
            ],
            'setting_is_sandbox' => [
                'id' => 'setting_is_sandbox',
                'type' => 'select',
                'name' => 'setting_is_sandbox',
                'label' => 'Sandbox Mode',
                'source' => 'yesno',
                'help' => 'Do you want to use test environment?'
            ],
            'setting_login' => [
                'id' => 'setting_login',
                'type' => 'text',
                'name' => 'setting_login',
                'label' => 'Login',
                'help' => 'Type login that you use',
                'required' => true
            ],
            'setting_password' => [
                'id' => 'setting_password',
                'type' => 'password',
                'name' => 'setting_password',
                'label' => 'Password',
                'help' => 'Type here password generated from allegro/aukro',
                'disabled' => true
            ],
            'setting_webapi_key' => [
                'id' => 'setting_webapi_key',
                'type' => 'text',
                'name' => 'setting_webapi_key',
                'label' => 'WebAPI Key',
                'help' => 'Type here your webapi key from allegro/aukro',
                'disabled' => true
            ],
            'setting_client_id' => [
                'id' => 'setting_client_id',
                'type' => 'text',
                'name' => 'setting_client_id',
                'label' => 'Client ID',
                'help' => 'Get Client ID by clicking on &#34;Create App&#34; button',
                'disabled' => true
            ],
            'setting_client_secret' => [
                'id' => 'setting_client_secret',
                'type' => 'text',
                'name' => 'setting_client_secret',
                'label' => 'Client Secret',
                'help' => 'Get Client Secret by clicking on &#34;Create App&#34; button',
                'disabled' => true
            ],
            'setting_client_token' => [
                'type' => 'text',
                'name' => 'setting_client_token',
                'label' => 'Token',
                'disabled' => true,
                'help' => 'Get Token by click on &#34;Connect&#34; button, Connect button will show when you fill Client ID and Secret'
            ],
            'save' => [
                'type' => 'submit',
                'name' => 'save',
                'label' => 'Save'
            ]
        ];
    }

    public function isConnectedApi($data)
    {
        if (is_null($data['setting_client_token'])) {
            return false;
        }

        return true;
    }

    public function isExpiredToken($data)
    {
        if (strtotime($data['setting_client_token_expires_at']) < time()) {
            return true;
        }

        return false;
    }
    
    public function getWebApiClient(){
        return $this->webapiClient;
    }
     
    public function checkWebAPIConnection($data){
        $webApiLib = GJMAA::getLib('webapi');
        $webApiLib->setCountry($data['setting_site']);
        $webApiLib->connectByLogin($data['setting_login'],$data['setting_password'],$data['setting_webapi_key']);
        if($error = $webApiLib->getError()){
            error_log($error);
            return false;
        } else {
            $this->webapiClient = $webApiLib;
            return true;
        }
    }
    
    public function refreshToken($settings){
        $restLibConnect = GJMAA::getLib('rest_api_auth_refresh');
        $restLibConnect->setSandboxMode($settings->getData('setting_is_sandbox'));
        $restLibConnect->setClientId($settings->getData('setting_client_id'));
        $restLibConnect->setClientSecret($settings->getData('setting_client_secret'));
        $restLibConnect->settRefreshToken($settings->getData('setting_client_refresh_token'));
        $restLibConnect->setRedirectUri(admin_url('admin.php?page=gjmaa_settings&action=authorisedCode'));
        $response = $restLibConnect->execute();
        
        $settings->setData('setting_client_token', $response['token']);
        $settings->setData('setting_client_token_expires_at', date('Y-m-d H:i:s', time() + $response['expiresIn']));
        $settings->setData('setting_client_refresh_token', $response['refreshToken']);
        $settings->save();
        
        return $settings;
    }
    
    public function getCategoriesFromWebAPI($settingsModel){
        $countryId = $settingsModel->getData('setting_site');
        
        $isConnected = $this->checkWebAPIConnection($settingsModel->getData());
        if ($isConnected) {
            $apiCategories = $this->getWebApiClient()->getCategories();
            if ($apiCategories) {
                $verKey = $apiCategories->verKey;
                $currentVer = get_option('gjmaa_category_' . $countryId,false);
                if (! $currentVer || $currentVer != $verKey) {
                    $categoryTree = [];
                    $catsList = is_array($apiCategories->catsList->item) ? $apiCategories->catsList->item : [
                        $apiCategories->catsList->item
                    ];
                    
                    
                    foreach ($catsList as $apiCategory) {
                        if (! isset($categoryTree[$apiCategory->catId])) {
                            $categoryTree[$apiCategory->catId] = [];
                        }
                        
                        $categoryTree[$apiCategory->catId] = [
                            'category_id' => $apiCategory->catId,
                            'category_parent_id' => $apiCategory->catParent,
                            'name' => $apiCategory->catName,
                            'country_id' => $countryId
                        ];
                    }
                    
                    $categoriesModel = GJMAA::getModel('allegro_category');
                    $categoriesModel->saveFullTree($categoryTree);
                    
                    add_option('gjmaa_category_' . $countryId, $verKey);
                }
            }
        }
    }
    
    public function isSandbox($settingId){
        $settings = GJMAA::getModel('settings')->load($settingId);
        
        return $settings->getData('setting_is_sandbox');
    }
}
?>