<?php

class GJMAA_Model_Settings extends GJMAA_Model
{

    protected $tableName = 'gjmaa_settings';

    protected $oldTableName = 'gj_allegro_settings';

    protected $defaultPK = 'setting_id';

    protected $mapColumns = [
        'allegro_api' => 'setting_webapi_key',
        'allegro_username' => 'setting_login',
        'allegro_password' => 'setting_password',
        'allegro_site' => 'setting_site',
        'allegro_sync' => 'setting_sync'
    ];

    protected $mapAsOption = [
        'last_category_check_1',
        'last_category_check_56',
        'category_version_1',
        'category_version_56'
    ];

    protected $columns = [
        'setting_id' => [
            'schema' => [
                'INT',
                'AUTO_INCREMENT',
                'NOT NULL'
            ],
            'format' => '%d'
        ],
        'setting_name' => [
            'schema' => [
                'VARCHAR (255)',
                'NOT NULL'
            ],
            'format' => '%s'
        ],
        'setting_site' => [
            'schema' => [
                'VARCHAR (255)',
                'NOT NULL'
            ],
            'format' => '%s'
        ],
        'setting_is_sandbox' => [
            'schema' => [
                'INT (2)',
                'NULL',
                'DEFAULT 0'
            ],
            'format' => '%d'
        ],
        'setting_webapi_key' => [
            'schema' => [
                'VARCHAR (255)',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_login' => [
            'schema' => [
                'VARCHAR (255)',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_password' => [
            'schema' => [
                'VARCHAR (255)',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_client_id' => [
            'schema' => [
                'VARCHAR (255)',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_client_secret' => [
            'schema' => [
                'VARCHAR (255)',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_client_token' => [
            'schema' => [
                'TEXT',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_client_token_expires_at' => [
            'schema' => [
                'DATETIME',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_client_refresh_token' => [
            'schema' => [
                'TEXT',
                'NULL'
            ],
            'format' => '%s'
        ],
        'setting_sync' => [
            'schema' => [
                'INT',
                'NULL',
                'DEFAULT 1'
            ],
            'format' => '%d'
        ]
    ];

    public function update($currentVersion)
    {
        if(version_compare($currentVersion,'2.0.0') < 0){
            $this->migerateFromOldToNew();
        }
        
        return $this;
    }

    public function migerateFromOldToNew()
    {
        $this->install();
        $mainTable = $this->tableName;
        $this->tableName = $this->oldTableName;
        $allEntries = $this->getAll();
        $this->tableName = $mainTable;

        foreach ($allEntries as $key => $config) {
            if (isset($this->mapColumns[$config['key']])) {
                $this->setData($this->mapColumns[$config['key']], $config['value']);
            }

            if (in_array($config['key'], $this->mapAsOption)) {
                if (! get_option('gjmaa_' . $config['key'], false)) {
                    add_option('gjmaa_' . $config['key'], $config['value']);
                } else {
                    update_option('gjmaa_' . $config['key'], $config['value']);
                }
            }
        }

        $this->setData('setting_name', 'Migrated Account');

        $this->save();
        return true;
    }
    
    public function getFirstLive(){
        $query = sprintf(
            "SELECT * FROM %s WHERE setting_is_sandbox <> %d",
            $this->getTable(),
            1
        );
        
        $setting = $this->wpdb->get_row($query, ARRAY_A);
        
        $this->setData($setting);
        
        return $this;
    }
}

?>