<?php

class GJMAA_Service_Categories
{

    protected $settings;

    protected $client;

    public function makeRequest()
    {
        if (! $this->client) {
            $api = GJMAA::getLib('rest_api_sale_categories');
            $this->client = $api;
        }
    }

    public function sendRequest()
    {
        $this->client->setToken($this->getSettings()
            ->getData('setting_client_token'));
        $this->client->setSandboxMode($this->getSettings()
            ->getData('setting_is_sandbox'));
        return $this->client->execute();
    }

    public function getCategoryByIdOrParentId($categoryId = null, $parentCategoryId = null)
    {
        if ($this->connect()) {
            $this->makeRequest();
            $this->client->setCategoryId($categoryId);
            $this->client->setParentCategoryId($parentCategoryId);
            $response = $this-> sendRequest();
            return $this->parseResponse($response);
        } else {
            $settingsHelper = GJMAA::getHelper('settings');
            $settingsHelper->getCategoriesFromWebAPI($this->getSettings());
        }
        return [];
    }

    public function getFullTreeForCategory($categoryId)
    {
        $tree = [];
        $categories = $this->getCategoryByIdOrParentId($categoryId);
        $tree[] = $categories[0];
        if ($categories[0]['category_parent_id']) {
            do {
                $categories = $this->getCategoryByIdOrParentId($categories[0]['category_parent_id']);
                $tree[] = $categories[0];
            } while ($categories[0]['category_parent_id'] != 0);
        }
        
        return $tree;
    }

    public function getSettings()
    {
        return $this->settings;
    }
    
    public function setSettings($settings){
        $this->settings = $settings;
    }

    public function connect()
    {
        $settings = $this->getSettings(); 
        if(!$settings){
            return false;
        }

        $helper = GJMAA::getHelper('settings');

        if (! $helper->isConnectedApi($settings->getData())) {
            return false;
        }

        if ($helper->isExpiredToken($settings->getData())) {
            $this->refreshToken($settings);
        }

        return true;
    }

    public function refreshToken($settings)
    {
        $restLibConnect = GJMAA::getLib('rest_api_auth_refresh');
        $restLibConnect->setSandboxMode($settings->getData('setting_is_sandbox'));
        $restLibConnect->setClientId($settings->getData('setting_client_id'));
        $restLibConnect->setClientSecret($settings->getData('setting_client_secret'));
        $restLibConnect->settRefreshToken($settings->getData('setting_client_refresh_token'));
        $restLibConnect->setRedirectUri(admin_url('admin.php?page=gjmaa_settings&action=authorisedCode'));
        $response = $restLibConnect->execute();

        $settings->setData('setting_client_token', $response['token']);
        $settings->setData('setting_client_token_expires_at', date('Y-m-d H:i:s', time() + $response['expiresIn']));
        $settings->setData('setting_client_refresh_token', $response['refreshToken']);
        $settings->save();

        return $settings;
    }

    public function parseResponse($response)
    {
        $allegroCategories = isset($response['categories']) ? $response['categories'] : [
            $response
        ];
        $categories = [];
        foreach ($allegroCategories as $category) {
            $categories[] = [
                'category_id' => $category['id'],
                'category_parent_id' => $category['parent']['id'] ?: 0,
                'name' => $category['name'],
                'country_id' => $this->getSettings()->getData('setting_site')
            ];
        }
        return $categories;
    }
}
?>