<?php

class GJMAA_Service_Woocommerce
{

    protected $_isEnabled;

    protected $_wooCommerceMeta = [
        '_visibility' => 'visible',
        '_stock_status' => 'instock',
        'total_sales' => '0',
        '_downloadable' => 'no',
        '_virtual' => 'no',
        '_featured' => 'no',
        '_sku' => 'sku',
        '_price' => 'price',
        '_regular_price' => 'price',
        '_manage_stock' => 'yes',
        '_backorders' => 'no',
        '_stock' => 'quantity',
        '_allegro' => 'yes',
        '_allegro_auction_id' => 'itId'
    ];

    protected $_update = [
        '_stock',
        '_stock_status'
    ];

    public function isEnabled()
    {
        if (null === $this->_isEnabled) {
            if (function_exists('is_plugin_active')) {
                $this->_isEnabled = is_plugin_active('woocommerce/woocommerce.php');
            } else {
                include_once (ABSPATH . 'wp-admin/includes/plugin.php');
                $this->_isEnabled = is_plugin_active('woocommerce/woocommerce.php');
            }
        }
        return $this->_isEnabled;
    }

    public function saveProducts($auctionDetails)
    {
        ini_set('max_execution_time', 240);
        foreach ($auctionDetails as $index => $auction) {
            $this->addProduct($auction);
        }
    }

    public function addProduct($allegroProduct)
    {
        $categories = $allegroProduct->itemCats->item;
        $attributes = $allegroProduct->itemAttribs->item;
        $product = $allegroProduct->itemInfo;
        $post = $this->getProductIdOrCreate($allegroProduct);

        $post_id = $post['postId'];
        $media = $this->getProductImage($allegroProduct->itemImages->item);
        foreach ($this->_wooCommerceMeta as $meta_key => $meta_value) {
            $originalValue = get_post_meta($post_id, $meta_key, true);
            switch ($meta_key) :
                case '_sku':
                    $meta_value = (isset($allegroProduct->itEan) ? $allegroProduct->itEan : $product->itId);
                    break;
                case '_price':
                case '_regular_price':
                    $meta_value = ($product->itBuyNowActive ? $product->itBuyNowPrice : $product->itPrice);
                    break;
                case '_stock':
                    $meta_value = $product->itQuantity;
                    break;
                case '_stock_status':
                    $meta_value = $product->itQuantity > 0 ? 'instock' : 'outofstock';
                    break;
                case '_allegro_auction_id':
                    $meta_value = $product->itId;
                    break;
            endswitch
            ;
            if (! $originalValue || $post['new'] || in_array($meta_key, $this->_update)) {
                update_post_meta($post_id, $meta_key, $meta_value);
            }
        }

        $this->assignCategories($categories, $post_id);
        $this->assignMediaProduct($product, $media, $post_id);
        $this->assignAttributes($product, $attributes, $post_id);
    }

    public function getPriceData($auction)
    {
        $cPrice = null;
        $prices = isset($auction->priceInfo) ? $auction->priceInfo->item : $auction->itemPrice;
        foreach ($prices as $index => $price) {
            if ($price->priceType == 'buyNow' || $price->priceType == 1) {
                $cPrice = (float) $price->priceValue;
                break;
            } else
                $cPrice = (float) $price->priceValue;
        }
        return $cPrice;
    }

    public function getProductImage($images)
    {
        $images = is_array($images) ? $images : [
            $images
        ];
        $media = [];
        foreach ($images as $image) {
            if (! isset($media[$image->imageType])) {
                $media[$image->imageType] = [];
            }
            $media[$image->imageType][] = $image->imageUrl;
        }
        return $media;
    }

    public function attach_image($fileurl, $filealt, $type, $post_id)
    {
        $filename = str_replace(' ', '_', $filealt); // Get the filename including extension from the $fileurl e.g. myimage.jp
        $destination = WP_CONTENT_DIR . '/uploads/allegro'; // Specify where we wish to upload the file, generally in the wp uploads directory

        if (! is_dir($destination)) {
            mkdir($destination);
        }

        $destinationPath = $destination . '/' . $filename . '_' . $type . '.jpg';

        copy($fileurl, $destinationPath);

        $filetype = wp_check_filetype($destinationPath); // Get the mime type of the file

        $attachment = array( // Set up our images post data
            'guid' => get_option('siteurl') . '/wp-content/uploads/allegro/' . $filename . '_' . $type . 'jpg',
            'post_mime_type' => $filetype['type'],
            'post_title' => $filename . '_' . $type . '.jpg',
            'post_author' => 1,
            'post_content' => ''
        );

        $attach_id = wp_insert_attachment($attachment, $destinationPath, $post_id); // Attach/upload image to the specified post id, think of this as adding a new post.

        if (! function_exists('wp_generate_attachment_metadata')) {
            include_once (ABSPATH . 'wp-admin/includes/image.php');
        }

        $attach_data = wp_generate_attachment_metadata($attach_id, $destinationPath); // Generate the necessary attachment data, filesize, height, width etc.

        wp_update_attachment_metadata($attach_id, $attach_data); // Add the above meta data data to our new image post

        add_post_meta($attach_id, '_wp_attachment_image_alt', $filealt); // Add the alt text to our new image post

        return $attach_id; // Return the images id to use in the below functions
    }

    public function getProductIdOrCreate($product)
    {
        $postId = wc_get_product_id_by_sku(isset($product->itEan) ? $product->itEan : $product->itemInfo->itId);
        if (0 === $postId) {
            $slug = sanitize_title($product->itemInfo->itName);
            if ($post = get_page_by_path($slug, OBJECT, 'product')) {
                $postId = $post->ID; 
            }

            $slugNew = $slug . '-' . $product->itemInfo->itId;

            if ($postId === 0) {
                if ($post = get_page_by_path($slugNew, OBJECT, 'product')) {
                    $postId = $post->ID;
                }
            } else {
                wp_update_post(array(
                    'ID' => $postId,
                    'post_name' => $slugNew
                ));
            }
        }
        $new = false;
        if (0 === $postId) {
            $slugNew = $slug . '-' . $product->itemInfo->itId;
            $postId = wp_insert_post(array(
                'post_title' => $product->itemInfo->itName,
                'post_content' => $this->stripTags($product->itemInfo->itDescription),
                'post_status' => 'publish',
                'post_type' => "product",
                'post_name' => $slugNew
            ));
            $new = true;
        }

        return [
            'postId' => $postId,
            'new' => $new
        ];
    }

    public function assignCategories($categories, $product_id)
    {
        $mapCategorySettings = GJMAA::getModel('categorymap');

        $categoriesWoo = $categoriesWooIds = $categoriesId = [];

        foreach ($categories as $category) {
            $categoriesId[$category->catId] = [
                'level' => $category->catLevel,
                'name' => $category->catName
            ];
        }

        $allegroIds = array_keys($categoriesId);
        if (! empty($allegroIds)) {
            $categoriesWoo = $mapCategorySettings->getMappedWooCommerceCategoriesByCategoryIds($allegroIds);
        }

        if (! empty($categoriesWoo)) {
            foreach ($categoriesWoo as $categoryWoo) {
                $categoriesWooIds[] = (int) $categoryWoo['woocommerce_category_id'];
            }
        }

        if (empty($categoriesWooIds)) {
            $categoriesWooIds = $this->addNewCategories($categories);
        }

        wp_set_object_terms($product_id, $categoriesWooIds, 'product_cat');
    }

    public function addNewCategories($categories)
    {
        $categoriesId = [];
        foreach ($categories as $category) {
            $term = wp_insert_term($category->catName, 'product_cat', [
                'description' => $category->catName . ' (' . $category->catId . ')',
                'slug' => strtolower(str_replace(' ', '-', $category->catName)),
                'parent' => $category->catLevel > 0 ? $categoriesId[$category->catLevel - 1] : 0
            ]);

            if (is_wp_error($term)) {
                $term_id = $term->error_data['term_exists'] ?: null;
            } else {
                $term_id = $term['term_id'];
            }

            if (null !== $term_id) {
                $categoriesId[$category->catLevel] = $term_id;
            }
        }

        return $categoriesId;
    }

    public function assignMediaProduct($product, $media, $product_id)
    {
        if (! get_post_meta($product_id, '_thumbnail_id')) {
            $attachment_ids = null;

            foreach ($media as $type => $image) {
                if ($type === 3) {
                    $attachment_ids = $this->attach_image($image[0], $product->itId, 0, $product_id);
                    break;
                }
            }

            if ($attachment_ids) {
                update_post_meta($product_id, '_thumbnail_id', $attachment_ids);
            }
        }

        if (! get_post_meta($product_id, '_product_image_gallery') && isset($media[3]) && count($media[3]) > 1) {
            $product_images = [];
            foreach ($media[3] as $type => $image) {
                if ($type > 0) {
                    $product_images[] = $this->attach_image($image, $product->itId, $type, $product_id);
                }
            }

            if ($product_images) {
                update_post_meta($product_id, '_product_image_gallery', implode(',', $product_images));
            }
        }
    }

    public function assignAttributes($product, $attributes, $product_id)
    {
        $wooCommerceAttributes = [];

        $attributes = is_array($attributes) ? $attributes : [
            $attributes
        ];

        foreach ($attributes as $attribute) {
            $attributeName = $attribute->attribName;
            $values = $attribute->attribValues->item;
            $attributeValue = is_array($values) ? implode(',', $values) : $values;
            $index = 'pa_' . str_replace(' ', '_', strtolower((string) $attributeName));
            $wooCommerceAttributes[$index] = [
                'name' => $attributeName,
                'value' => $attributeValue,
                'is_visible' => 1,
                'is_taxonomy' => 0
            ];
        }

        update_post_meta($product_id, '_product_attributes', $wooCommerceAttributes);
    }

    public function showOnAllegro($ID)
    {
        $sku = get_post_meta($ID, '_sku', true);

        $auctionsAllegro = new GjmaaAuctionsItem();
        $auctions = $auctionsAllegro->getById($sku);

        $gjmaaSettings = new GjmaaMyAuctionsAllegro();

        $settingsId = isset($auctions[$sku]) ? $auctions[$sku] : null;
        if ($settingsId) {
            $settingsData = $gjmaaSettings->getById($settingsId);
            $url = 'https://' . $gjmaaSettings->getNameOfSite($settingsData['site_allegro']) . '/show_item.php?item=' . $sku;
        } else {
            $url = '#';
        }

        return $url;
    }

    public function stripTags($description)
    {
        $description = preg_replace(array(
            // Remove invisible content
            '@<head[^>]*?>.*?</head>@siu',
            '@<style[^>]*?>.*?</style>@siu',
            '@<script[^>]*?.*?</script>@siu',
            '@<object[^>]*?.*?</object>@siu',
            '@<embed[^>]*?.*?</embed>@siu',
            '@<applet[^>]*?.*?</applet>@siu',
            '@<noframes[^>]*?.*?</noframes>@siu',
            '@<noscript[^>]*?.*?</noscript>@siu',
            '@<noembed[^>]*?.*?</noembed>@siu'
            // Add line breaks before and after blocks
        ), array(
            ' ',
            ' ',
            ' ',
            ' ',
            ' ',
            ' ',
            ' ',
            ' ',
            ' '
        ), $description);

        return strip_tags($description, '<p><img><a><h1><h2><h3><ul><li><ol><table><tbody><thead><tr><th><td>');
    }

    public function getCategories($parentId = 0)
    {
        $args = array(
            'hierarchical' => 1,
            'show_option_none' => '',
            'hide_empty' => 0,
            'parent' => $parentId,
            'taxonomy' => 'product_cat'
        );

        $select = [];

        $currentCategory = false;

        if ($parentId) {
            $currentCategory = get_term_by('id', $parentId, 'product_cat');
        }

        $categories = get_categories($args);
        if ($currentCategory) {
            $select[$currentCategory->parent] = ' <= ' . __('Back', 'my-auctions-allegro-free-edition');
            $select[$currentCategory->term_id] = $currentCategory->name;
        }
        foreach ($categories as $category) {
            $select[$category->term_id] = $currentCategory ? '-- ' . $category->name : $category->name;
        }

        return $select;
    }

    public function updateStock($stockData)
    {
        foreach ($stockData as $sku => $quantity) {
            $productId = wc_get_product_id_by_sku($sku);
            if (0 === $productId) {
                continue;
            }

            update_post_meta($productId, '_stock', $quantity);
            update_post_meta($productId, '_stock_status', $quantity > 0 ? 'instock' : 'outofstock');
        }
    }
}